@@ -222,25 +222,6 @@ class MediaCategoryViewController: UICollectionViewController, UISearchBarDelega
         navItemTitle.font = UIFont.preferredCustomFont(forTextStyle: .headline).bolded
 
         self.navigationItem.titleView = navItemTitle
-        NotificationCenter.default.addObserver(self, selector: #selector(themeDidChange),
-                                               name: .VLCThemeDidChangeNotification, object: nil)
-
-        NotificationCenter.default.addObserver(self, selector: #selector(miniPlayerIsShown),
-                                               name: NSNotification.Name(rawValue: VLCPlayerDisplayControllerDisplayMiniPlayer),
-                                               object: nil)
-        NotificationCenter.default.addObserver(self, selector: #selector(miniPlayerIsHidden),
-                                               name: NSNotification.Name(rawValue: VLCPlayerDisplayControllerHideMiniPlayer),
-                                               object: nil)
-        NotificationCenter.default.addObserver(self, selector: #selector(preferredContentSizeChanged(_:)),
-                                               name: UIContentSizeCategory.didChangeNotification,
-                                               object: nil)
-
-        if model is MediaGroupViewModel || model is VideoModel {
-            NotificationCenter.default.addObserver(self, selector: #selector(handleDisableGrouping),
-                                                   name: .VLCDisableGroupingDidChangeNotification,
-                                                   object: nil)
-        }
-
 
     }
 
@@ -441,6 +422,8 @@ class MediaCategoryViewController: UICollectionViewController, UISearchBarDelega
         setupCollectionView() //Fixes crash that is caused due to layout change
         setNavbarAppearance()
         loadSort()
+        
+        addInitializationCommonObservers()
     }
 
     override func viewDidAppear(_ animated: Bool) {
@@ -448,6 +431,47 @@ class MediaCategoryViewController: UICollectionViewController, UISearchBarDelega
         updateCollectionViewForAlbum()
     }
 
+    override func viewWillDisappear(_ animated: Bool) {
+        super.viewWillDisappear(animated)
+        removeInitializationCommonObservers()
+    }
+    
+    private func addInitializationCommonObservers() {
+        let notificationCenter = NotificationCenter.default
+
+        notificationCenter.addObserver(self, selector: #selector(themeDidChange),
+                                               name: .VLCThemeDidChangeNotification, object: nil)
+
+        notificationCenter.addObserver(self, selector: #selector(miniPlayerIsShown),
+                                               name: NSNotification.Name(rawValue: VLCPlayerDisplayControllerDisplayMiniPlayer),
+                                               object: nil)
+        notificationCenter.addObserver(self, selector: #selector(miniPlayerIsHidden),
+                                               name: NSNotification.Name(rawValue: VLCPlayerDisplayControllerHideMiniPlayer),
+                                               object: nil)
+        notificationCenter.addObserver(self, selector: #selector(preferredContentSizeChanged(_:)),
+                                               name: UIContentSizeCategory.didChangeNotification,
+                                               object: nil)
+
+        if model is MediaGroupViewModel || model is VideoModel {
+            notificationCenter.addObserver(self, selector: #selector(handleDisableGrouping),
+                                                   name: .VLCDisableGroupingDidChangeNotification,
+                                                   object: nil)
+        }
+    }
+    
+    private func removeInitializationCommonObservers() {
+        let notificationCenter = NotificationCenter.default
+
+        notificationCenter.removeObserver(self, name: .VLCThemeDidChangeNotification, object: nil)
+        notificationCenter.removeObserver(self, name: NSNotification.Name(rawValue: VLCPlayerDisplayControllerDisplayMiniPlayer), object: nil)
+        notificationCenter.removeObserver(self, name:  NSNotification.Name(rawValue: VLCPlayerDisplayControllerHideMiniPlayer), object: nil)
+        notificationCenter.removeObserver(self, name: UIContentSizeCategory.didChangeNotification, object: nil)
+
+        if model is MediaGroupViewModel || model is VideoModel {
+            notificationCenter.removeObserver(self, name: .VLCDisableGroupingDidChangeNotification, object: nil)
+        }
+    }
+
     func loadSort() {
         let sortingCriteria: VLCMLSortingCriteria
         if let sortingCriteriaDefault = UserDefaults.standard.value(forKey: "\(kVLCSortDefault)\(model.name)") as? UInt {
