@@ -12,6 +12,7 @@
  *          Vincent L. Cone <vincent.l.cone # tuta.io>
  *          Carola Nitz <caro # videolan.org>
  *          Diogo Simao Marques <dogo@videolabs.io>
+ *          Eshan Singh <eeeshan789@gmail.com>
  *
  * Refer to the COPYING file of the official project for license.
  *****************************************************************************/
@@ -59,6 +60,8 @@
     VLCRemoteNetworkDataSourceAndDelegate *_remoteNetworkDataSourceAndDelegate;
     NSLayoutConstraint* _localNetworkHeight;
     NSLayoutConstraint* _remoteNetworkHeight;
+    MediaLibraryService *_medialibraryService;
+    UIDocumentPickerViewController *_documentPicker;
 }
 
 @end
@@ -86,10 +89,11 @@
 
 #endif
 
-- (instancetype)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
+- (instancetype)initWithMedialibraryService:(MediaLibraryService *)medialibraryService
 {
-    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
+    self = [super initWithNibName:nil bundle:nil];
     if (self) {
+        _medialibraryService = medialibraryService;
         [self setupUI];
     }
     return self;
@@ -104,11 +108,11 @@
     [self.view addSubview:_scrollView];
 
     [NSLayoutConstraint activateConstraints:@[
-                                              [_scrollView.leftAnchor constraintEqualToAnchor:self.view.leftAnchor],
-                                              [_scrollView.rightAnchor constraintEqualToAnchor:self.view.rightAnchor],
-                                              [_scrollView.topAnchor constraintEqualToAnchor:self.topLayoutGuide.bottomAnchor],
-                                              [_scrollView.bottomAnchor constraintEqualToAnchor:self.view.bottomAnchor],
-                                              ]];
+        [_scrollView.leftAnchor constraintEqualToAnchor:self.view.leftAnchor],
+        [_scrollView.rightAnchor constraintEqualToAnchor:self.view.rightAnchor],
+        [_scrollView.topAnchor constraintEqualToAnchor:self.topLayoutGuide.bottomAnchor],
+        [_scrollView.bottomAnchor constraintEqualToAnchor:self.view.bottomAnchor],
+    ]];
 
     _remoteNetworkDataSourceAndDelegate = [VLCRemoteNetworkDataSourceAndDelegate new];
     _remoteNetworkDataSourceAndDelegate.delegate = self;
@@ -165,19 +169,19 @@
     _remoteNetworkHeight = [_remoteNetworkTableView.heightAnchor constraintEqualToConstant:_remoteNetworkTableView.contentSize.height];
 
     [NSLayoutConstraint activateConstraints:@[
-                                              [_remoteNetworkTableView.leftAnchor constraintEqualToAnchor:self.view.leftAnchor],
-                                              [_remoteNetworkTableView.rightAnchor constraintEqualToAnchor:self.view.rightAnchor],
-                                              [_remoteNetworkTableView.topAnchor constraintEqualToAnchor:_scrollView.topAnchor],
-                                              [fileServerView.topAnchor constraintEqualToAnchor:_remoteNetworkTableView.bottomAnchor],
-                                              [fileServerView.leftAnchor constraintEqualToAnchor:self.view.leftAnchor],
-                                              [fileServerView.rightAnchor constraintEqualToAnchor:self.view.rightAnchor],
-                                              [_localNetworkTableView.topAnchor constraintEqualToAnchor:fileServerView.bottomAnchor],
-                                              [_localNetworkTableView.leftAnchor constraintEqualToAnchor:self.view.leftAnchor],
-                                              [_localNetworkTableView.rightAnchor constraintEqualToAnchor:self.view.rightAnchor],
-                                              [_localNetworkTableView.bottomAnchor constraintEqualToAnchor:_scrollView.bottomAnchor],
-                                              _localNetworkHeight,
-                                              _remoteNetworkHeight
-                                              ]];
+        [_remoteNetworkTableView.leftAnchor constraintEqualToAnchor:self.view.leftAnchor],
+        [_remoteNetworkTableView.rightAnchor constraintEqualToAnchor:self.view.rightAnchor],
+        [_remoteNetworkTableView.topAnchor constraintEqualToAnchor:_scrollView.topAnchor],
+        [fileServerView.topAnchor constraintEqualToAnchor:_remoteNetworkTableView.bottomAnchor],
+        [fileServerView.leftAnchor constraintEqualToAnchor:self.view.leftAnchor],
+        [fileServerView.rightAnchor constraintEqualToAnchor:self.view.rightAnchor],
+        [_localNetworkTableView.topAnchor constraintEqualToAnchor:fileServerView.bottomAnchor],
+        [_localNetworkTableView.leftAnchor constraintEqualToAnchor:self.view.leftAnchor],
+        [_localNetworkTableView.rightAnchor constraintEqualToAnchor:self.view.rightAnchor],
+        [_localNetworkTableView.bottomAnchor constraintEqualToAnchor:_scrollView.bottomAnchor],
+        _localNetworkHeight,
+        _remoteNetworkHeight
+    ]];
 }
 
 - (void)setupUI
@@ -201,6 +205,10 @@
 {
     [super viewDidLoad];
 
+    if (@available(iOS 11.0, *)) {
+        self.navigationController.navigationBar.prefersLargeTitles = NO;
+    }
+
     NSNotificationCenter *notificationCenter = [NSNotificationCenter defaultCenter];
     [notificationCenter addObserver:self selector:@selector(themeDidChange) name:kVLCThemeDidChangeNotification object:nil];
     [notificationCenter addObserver:self selector:@selector(contentSizeDidChange) name:UIContentSizeCategoryDidChangeNotification object:nil];
@@ -212,13 +220,13 @@
 
     [self themeDidChange];
     NSArray *browserClasses = @[
-                                [VLCLocalNetworkServiceBrowserUPnP class],
-                                [VLCLocalNetworkServiceBrowserPlex class],
-                                [VLCLocalNetworkServiceBrowserHTTP class],
-                                [VLCLocalNetworkServiceBrowserDSM class],
-                                [VLCLocalNetworkServiceBrowserBonjour class],
-                                [VLCLocalNetworkServiceBrowserNFS class],
-                                ];
+        [VLCLocalNetworkServiceBrowserUPnP class],
+        [VLCLocalNetworkServiceBrowserPlex class],
+        [VLCLocalNetworkServiceBrowserHTTP class],
+        [VLCLocalNetworkServiceBrowserDSM class],
+        [VLCLocalNetworkServiceBrowserBonjour class],
+        [VLCLocalNetworkServiceBrowserNFS class],
+    ];
 
     _discoveryController = [[VLCLocalServerDiscoveryController alloc] initWithServiceBrowserClasses:browserClasses];
     _discoveryController.delegate = self;
@@ -237,9 +245,6 @@
 - (void)viewWillAppear:(BOOL)animated
 {
     [super viewWillAppear:animated];
-    if (@available(iOS 11.0, *)) {
-        self.navigationController.navigationBar.prefersLargeTitles = YES;
-    }
     VLCPlaybackService.sharedInstance.playerDisplayController.isMiniPlayerVisible
     ? [self miniPlayerIsShown] : [self miniPlayerIsHidden];
     dispatch_async(dispatch_get_main_queue(), ^{
@@ -252,6 +257,7 @@
 {
     _remoteNetworkHeight.constant = _remoteNetworkTableView.contentSize.height;
     _localNetworkHeight.constant = _localNetworkTableView.contentSize.height;
+    [super viewDidAppear:animated];
 }
 
 - (void)miniPlayerIsShown
@@ -370,7 +376,9 @@
     if ([service respondsToSelector:@selector(serverBrowser)]) {
         id<VLCNetworkServerBrowser> serverBrowser = [service serverBrowser];
         if (serverBrowser) {
-            VLCNetworkServerBrowserViewController *vc = [[VLCNetworkServerBrowserViewController alloc] initWithServerBrowser:serverBrowser];
+            VLCNetworkServerBrowserViewController *vc = [[VLCNetworkServerBrowserViewController alloc]
+                                                         initWithServerBrowser:serverBrowser
+                                                         medialibraryService:_medialibraryService];
             [self.navigationController pushViewController:vc animated:YES];
             return;
         }
@@ -395,13 +403,24 @@
 
     /* UPnP does not support authentication, so skip this step */
     if ([login.protocolIdentifier isEqualToString:VLCNetworkServerProtocolIdentifierUPnP]) {
-        VLCNetworkServerBrowserVLCMedia *serverBrowser = [VLCNetworkServerBrowserVLCMedia UPnPNetworkServerBrowserWithLogin:login];
-        VLCNetworkServerBrowserViewController *vc = [[VLCNetworkServerBrowserViewController alloc] initWithServerBrowser:serverBrowser];
+        VLCNetworkServerBrowserVLCMedia *serverBrowser;
+        if (login.rootMedia != nil) {
+            serverBrowser = [[VLCNetworkServerBrowserVLCMedia alloc] initWithMedia:login.rootMedia options:login.options];
+        } else {
+            serverBrowser = [VLCNetworkServerBrowserVLCMedia UPnPNetworkServerBrowserWithLogin:login];
+        }
+        VLCNetworkServerBrowserViewController *vc = [[VLCNetworkServerBrowserViewController alloc]
+                                                     initWithServerBrowser:serverBrowser
+                                                     medialibraryService:_medialibraryService];
         [self.navigationController pushViewController:vc animated:YES];
         return;
     }
 
-    [login loadLoginInformationFromKeychainWithError:nil];
+    NSError *error = nil;
+    if (![login loadLoginInformationFromKeychainWithError:&error]) {
+        [self showKeychainLoadError:error forLogin:login];
+        return;
+    }
 
     VLCNetworkLoginViewController *loginViewController = [[VLCNetworkLoginViewController alloc] initWithNibName:@"VLCNetworkLoginViewController" bundle:nil];
 
@@ -448,6 +467,18 @@
     }
 }
 
+- (void)showKeychainLoadError:(NSError *)error forLogin:(VLCNetworkServerLoginInformation *)login
+{
+    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:error.localizedDescription
+                                                                             message:error.localizedFailureReason preferredStyle:UIAlertControllerStyleAlert];
+    [alertController addAction:[UIAlertAction actionWithTitle:NSLocalizedString(@"BUTTON_OK", nil)
+                                                        style:UIAlertActionStyleDefault
+                                                      handler:^(UIAlertAction * _Nonnull action) {
+        [self connectToServer];
+    }]];
+    [self presentViewController:alertController animated:YES completion:nil];
+}
+
 - (void)showViewController:(UIViewController *)viewController
 {
     [self.navigationController pushViewController:viewController animated:YES];
@@ -455,8 +486,14 @@
 
 - (void)showDocumentPickerViewController:(UIDocumentPickerViewController *)viewControllerToPresent
 {
-    viewControllerToPresent.delegate = self;
-    [self presentViewController:viewControllerToPresent animated:YES completion:nil];
+    _documentPicker = viewControllerToPresent;
+    _documentPicker.delegate = self;
+
+    if (@available(iOS 11.0, *)) {
+        _documentPicker.allowsMultipleSelection = YES;
+    }
+
+    [self presentViewController:_documentPicker animated:YES completion:nil];
 }
 
 - (void)reloadRemoteTableView
@@ -540,7 +577,9 @@
     }
 
     if (serverBrowser) {
-        VLCNetworkServerBrowserViewController *targetViewController = [[VLCNetworkServerBrowserViewController alloc] initWithServerBrowser:serverBrowser];
+        VLCNetworkServerBrowserViewController *targetViewController = [[VLCNetworkServerBrowserViewController alloc]
+                                                                       initWithServerBrowser:serverBrowser
+                                                                       medialibraryService:_medialibraryService];
         [self.navigationController pushViewController:targetViewController animated:YES];
     }
 }
@@ -552,6 +591,18 @@
     _localNetworkHeight.constant = _localNetworkTableView.contentSize.height;
 }
 
+-(void)showEmptyMediaListAlert
+{
+    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:NSLocalizedString(@"EMPTY_MEDIA_LIST", "")
+                                                                             message:NSLocalizedString(@"EMPTY_MEDIA_LIST_DESCRIPTION", "")
+                                                                      preferredStyle:UIAlertControllerStyleAlert];
+
+    UIAlertAction *dismissAction = [UIAlertAction actionWithTitle:NSLocalizedString(@"BUTTON_DISMISS", "") style:UIAlertActionStyleCancel handler:nil];
+
+    [alertController addAction:dismissAction];
+    [self presentViewController:alertController animated:YES completion:nil];
+}
+
 #pragma mark - UIDocumentPickerDelegate
 
 - (void)documentPicker:(UIDocumentPickerViewController *)controller didPickDocumentAtURL:(NSURL *)url
@@ -564,4 +615,53 @@
     }
 }
 
+- (void)documentPicker:(UIDocumentPickerViewController *)controller didPickDocumentsAtURLs:(NSArray<NSURL *> *)urls API_AVAILABLE(ios(11.0))
+{
+    if (urls.count > 0) {
+        VLCMediaList *medialist = [[VLCMediaList alloc] init];
+
+        if (urls.count == 1 && [[urls[0] pathExtension] isEqualToString:@""]) {
+            [self getFolderData:urls[0] mediaList:medialist];
+        } else {
+            for (NSURL *url in urls) {
+                NSString *pathExtension = [url pathExtension];
+                if (![pathExtension isEqualToString:@""]) {
+                    if (url && [url startAccessingSecurityScopedResource]) {
+                        [medialist addMedia:[VLCMedia mediaWithURL:url]];
+                        [[VLCPlaybackService sharedInstance].openedLocalURLs addObject:url];
+                    }
+                }
+            }
+        }
+
+        if ([medialist count] > 0) {
+            [[VLCPlaybackService sharedInstance] playMediaList:medialist firstIndex:0 subtitlesFilePath:nil];
+        } else {
+            [self showEmptyMediaListAlert];
+        }
+    }
+}
+
+-(void)getFolderData:(NSURL*)url mediaList:(VLCMediaList*) list
+{
+    NSURL *folderURL = url;
+    NSError *error = nil;
+    [url startAccessingSecurityScopedResource];
+    NSArray<NSURL *> *filesInFolder = [[NSFileManager defaultManager] contentsOfDirectoryAtURL:folderURL
+                                                                    includingPropertiesForKeys:@[]
+                                                                                       options:NSDirectoryEnumerationSkipsHiddenFiles
+                                                                                         error:&error];
+    if (error) {
+        NSLog(@"Error reading directory: %@", error);
+        return;
+    }
+
+    for (NSURL *fileURL in filesInFolder) {
+        if (![fileURL.pathExtension isEqual:@""]) {
+            [list addMedia:[VLCMedia mediaWithURL:fileURL]];
+            [[VLCPlaybackService sharedInstance].openedLocalURLs addObject:fileURL];
+        }
+    }
+}
+
 @end
