@@ -17,6 +17,8 @@
 #import "VLC-Swift.h"
 #if TARGET_OS_TV
 #import "VLCFullscreenMovieTVViewController.h"
+#else
+#import "UIStackView+Orientation.h"
 #endif
 
 #define SWIFT_VIDEO_PLAYER 1
@@ -82,6 +84,11 @@ NSString *const VLCPlayerDisplayControllerHideMiniPlayer = @"VLCPlayerDisplayCon
 
 #pragma mark - properties
 
+- (UIViewController *)videoPlayerViewController
+{
+    return _videoPlayerViewController;
+}
+
 - (VLCPlayerDisplayControllerDisplayMode)displayMode
 {
     return [[NSUserDefaults standardUserDefaults] integerForKey:VLCPlayerDisplayControllerDisplayModeKey];
@@ -162,6 +169,11 @@ NSString *const VLCPlayerDisplayControllerHideMiniPlayer = @"VLCPlayerDisplayCon
         [self setDisplayMode:VLCPlayerDisplayControllerDisplayModeFullscreen];
     }
 
+    if ([notification.userInfo[kVLCPlayerOpenInMiniPlayer] isEqual: @YES]) {
+        [self setDisplayMode:VLCPlayerDisplayControllerDisplayModeMiniplayer];
+        [self.playbackController pause];
+    }
+
     switch (self.displayMode) {
         case VLCPlayerDisplayControllerDisplayModeFullscreen:
             if ((media.type == VLCMLMediaTypeAudio || _playbackController.playAsAudio) &&
@@ -205,7 +217,9 @@ NSString *const VLCPlayerDisplayControllerHideMiniPlayer = @"VLCPlayerDisplayCon
 
 - (void)closeFullscreenPlayback
 {
-    [self.movieViewController dismissViewControllerAnimated:[self shouldAnimate] completion:nil];
+    [self.movieViewController dismissViewControllerAnimated:[self shouldAnimate] completion:^{
+        [self _updateInterfaceOrientation];
+    }];
     self.displayMode = VLCPlayerDisplayControllerDisplayModeMiniplayer;
     [self _showHideMiniPlaybackView];
 }
@@ -309,7 +323,9 @@ NSString *const VLCPlayerDisplayControllerHideMiniPlayer = @"VLCPlayerDisplayCon
 {
     dispatch_async(dispatch_get_main_queue(), ^{
         BOOL animated = [self shouldAnimate];
-        [self.movieViewController dismissViewControllerAnimated:animated completion:nil];
+        [self.movieViewController dismissViewControllerAnimated:animated completion:^{
+            [self _updateInterfaceOrientation];
+        }];
         [self _showHideMiniPlaybackView];
     });
 }
@@ -334,6 +350,7 @@ NSString *const VLCPlayerDisplayControllerHideMiniPlayer = @"VLCPlayerDisplayCon
     UINavigationController *navCon = [[VLCPlaybackNavigationController alloc] initWithRootViewController:audioPlayerViewController];
     navCon.modalPresentationStyle = UIModalPresentationOverFullScreen;
     navCon.modalPresentationCapturesStatusBarAppearance = YES;
+    _playbackController.delegate = audioPlayerViewController;
     [audioPlayerViewController prepareForMediaPlayback:self.playbackController];
 
     UIWindow *window = [[UIApplication sharedApplication] keyWindow];
@@ -342,6 +359,25 @@ NSString *const VLCPlayerDisplayControllerHideMiniPlayer = @"VLCPlayerDisplayCon
     }];
 }
 
+#if !TARGET_OS_TV
+- (void)_updateInterfaceOrientation
+{
+    if (@available(iOS 16, *)) {
+        UIWindowScene *windowScene = self.view.window.windowScene;
+        if (windowScene) {
+            id geometry = [[VLCWindowSceneGeometryPreferencesIOS alloc] initWithInterfaceOrientations:UIInterfaceOrientationMaskAllButUpsideDown];
+            [windowScene requestGeometryUpdateWithPreferences:geometry errorHandler:nil];
+            [self setNeedsUpdateOfSupportedInterfaceOrientations];
+        }
+    } else {
+        VLCPlayerDisplayController *vpdc = [[VLCPlaybackService sharedInstance] playerDisplayController];
+        VLCVideoPlayerViewController *videoVC = (VLCVideoPlayerViewController *)vpdc.videoPlayerViewController;
+        videoVC.supportedInterfaceOrientations = UIInterfaceOrientationMaskAllButUpsideDown;
+        [UIViewController attemptRotationToDeviceOrientation];
+    }
+}
+#endif
+
 #pragma mark - miniplayer
 
 - (BOOL)isMiniPlayerVisible
@@ -375,6 +411,7 @@ NSString *const VLCPlayerDisplayControllerHideMiniPlayer = @"VLCPlayerDisplayCon
 
     void (^completionBlock)(BOOL) = nil;
     if (needsShow) {
+        // Init the mini player view if needed
         if (!miniPlaybackView) {
             UIViewController *rootViewController = UIApplication.sharedApplication.keyWindow.rootViewController;
 
@@ -412,6 +449,15 @@ NSString *const VLCPlayerDisplayControllerHideMiniPlayer = @"VLCPlayerDisplayCon
             [((VLCAudioMiniPlayer*)_miniPlaybackView) setupQueueViewControllerWith:_queueViewController];
             [self.view layoutIfNeeded];
         }
+
+        // Properly set the shuffle and repeat mode
+        NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
+        if ([userDefaults boolForKey:kVLCPlayerShouldRememberState]) {
+            _playbackController.shuffleMode = [userDefaults boolForKey:kVLCPlayerIsShuffleEnabled];
+            NSInteger repeatMode = [userDefaults integerForKey:kVLCPlayerIsRepeatEnabled];
+            _playbackController.repeatMode = repeatMode;
+        }
+
         [self addPlayqueueToMiniPlayer];
         miniPlaybackView.visible = YES;
         [[NSNotificationCenter defaultCenter]
@@ -428,7 +474,6 @@ NSString *const VLCPlayerDisplayControllerHideMiniPlayer = @"VLCPlayerDisplayCon
             }
             [self->_queueViewController hide];
             [self->_queueViewController removeFromParentViewController];
-            [self resignFirstResponder];
         };
     }
     //when switching between tableview and collectionview all subviews are removed, make sure to readd it when this happens
