@@ -13,16 +13,19 @@ import UIKit
 
 protocol AudioPlayerViewDelegate: AnyObject {
     func audioPlayerViewDelegateGetThumbnail(_ audioPlayerView: AudioPlayerView) -> UIImage?
+    func audioPlayerViewDelegateGetPlaybackSpeed(_ audioPlayerView: AudioPlayerView) -> Float
     func audioPlayerViewDelegateDidTapShuffleButton(_ audioPlayerView: AudioPlayerView)
     func audioPlayerViewDelegateDidTapPreviousButton(_ audioPlayerView: AudioPlayerView)
     func audioPlayerViewDelegateDidTapPlayButton(_ audioPlayerView: AudioPlayerView)
     func audioPlayerViewDelegateDidTapNextButton(_ audioPlayerView: AudioPlayerView)
     func audioPlayerViewDelegateDidTapRepeatButton(_ audioPlayerView: AudioPlayerView)
+    func audioPlayerViewDelegateDidTapPlaybackSpeedButton(_ audioPlayerView: AudioPlayerView)
+    func audioPlayerViewDelegateDidLongPressPlaybackSpeedButton(_ audioPlayerView: AudioPlayerView)
     func audioPlayerViewDelegateGetBrightnessSlider(_ audioPlayerView: AudioPlayerView) -> BrightnessControlView
     func audioPlayerViewDelegateGetVolumeSlider(_ audioPlayerView: AudioPlayerView) -> VolumeControlView
 }
 
-class AudioPlayerView: UIView {
+class AudioPlayerView: UIView, UIGestureRecognizerDelegate {
     // MARK: - Properties
 
     private lazy var backgroundView: UIView = UIView()
@@ -43,6 +46,7 @@ class AudioPlayerView: UIView {
         let titleLabel = UILabel()
         titleLabel.textAlignment = .center
         titleLabel.font = .boldSystemFont(ofSize: 17.0)
+        titleLabel.accessibilityLabel = NSLocalizedString("TITLE", comment: "")
         return titleLabel
     }()
 
@@ -50,12 +54,14 @@ class AudioPlayerView: UIView {
         let artistLabel = UILabel()
         artistLabel.textAlignment = .center
         artistLabel.font = .systemFont(ofSize: 16.0)
+        artistLabel.accessibilityLabel = NSLocalizedString("ARTIST", comment: "")
         return artistLabel
     }()
 
     lazy var playqueueView: UIView = UIView()
 
     lazy var controlsStackView: UIStackView = UIStackView()
+    lazy var secondaryControlStackView: UIStackView = UIStackView()
 
     private lazy var shuffleButton: UIButton = {
         let shuffleButton = UIButton(type: .system)
@@ -64,6 +70,8 @@ class AudioPlayerView: UIView {
         shuffleButton.imageView?.contentMode = .scaleAspectFit
         shuffleButton.tintColor = .white
         shuffleButton.addTarget(self, action: #selector(handleShuffleButton(_:)), for: .touchUpInside)
+        shuffleButton.accessibilityLabel = NSLocalizedString("SHUFFLE", comment: "")
+        shuffleButton.accessibilityHint = NSLocalizedString("SHUFFLE_HINT", comment: "")
         return shuffleButton
     }()
 
@@ -74,8 +82,25 @@ class AudioPlayerView: UIView {
         previousButton.imageView?.contentMode = .scaleAspectFit
         previousButton.tintColor = .white
         previousButton.addTarget(self, action: #selector(handlePreviousButton(_:)), for: .touchUpInside)
+        previousButton.accessibilityLabel = NSLocalizedString("PREVIOUS_BUTTON", comment: "")
+        previousButton.accessibilityHint = NSLocalizedString("PREVIOUS_HINT", comment: "")
         return previousButton
     }()
+    
+    private lazy var playbackSpeedButton: UIButton = {
+        let playbackButton = UIButton(type: .system)
+        let longPressGestureRecognizer = UILongPressGestureRecognizer(target: self, action: #selector(handleLongPressPlaybackSpeedButton(_:)))
+        playbackButton.contentMode = .scaleAspectFit
+        playbackButton.imageView?.contentMode = .scaleAspectFit
+        playbackButton.tintColor = .white
+        playbackButton.titleLabel?.font = UIFont.systemFont(ofSize: 16, weight: .bold)
+        playbackButton.addTarget(self, action: #selector(handlePlaybackSpeedButton(_:)), for: .touchUpInside)
+        longPressGestureRecognizer.minimumPressDuration = 0.5
+        longPressGestureRecognizer.delaysTouchesBegan = true
+        longPressGestureRecognizer.delegate = self
+        playbackButton.addGestureRecognizer(longPressGestureRecognizer)
+        return playbackButton
+    }()
 
     private lazy var playButton: UIButton = {
         let playButton = UIButton()
@@ -84,6 +109,8 @@ class AudioPlayerView: UIView {
         playButton.imageView?.contentMode = .scaleAspectFit
         playButton.tintColor = .white
         playButton.addTarget(self, action: #selector(handlePlayButton(_:)), for: .touchUpInside)
+        playButton.accessibilityLabel = NSLocalizedString("PLAY_PAUSE_BUTTON", comment: "")
+        playButton.accessibilityHint = NSLocalizedString("PLAY_PAUSE_HINT", comment: "")
         return playButton
     }()
 
@@ -94,6 +121,8 @@ class AudioPlayerView: UIView {
         nextButton.imageView?.contentMode = .scaleAspectFit
         nextButton.tintColor = .white
         nextButton.addTarget(self, action: #selector(handleNextButton(_:)), for: .touchUpInside)
+        nextButton.accessibilityLabel = NSLocalizedString("NEXT_BUTTON", comment: "")
+        nextButton.accessibilityHint = NSLocalizedString("NEXT_HINT", comment: "")
         return nextButton
     }()
 
@@ -104,12 +133,14 @@ class AudioPlayerView: UIView {
         repeatButton.imageView?.contentMode = .scaleAspectFit
         repeatButton.tintColor = .white
         repeatButton.addTarget(self, action: #selector(handleRepeatButton(_:)), for: .touchUpInside)
+        repeatButton.accessibilityLabel = NSLocalizedString("REPEAT_MODE", comment: "")
+        repeatButton.accessibilityHint = NSLocalizedString("REPEAT_HINT", comment: "")
         return repeatButton
     }()
 
     lazy var progressionView: UIView = UIView()
 
-    private lazy var layoutGuide: UILayoutGuide = {
+    lazy var layoutGuide: UILayoutGuide = {
         var layoutGuide = layoutMarginsGuide
 
         if #available(iOS 11.0, *) {
@@ -126,10 +157,14 @@ class AudioPlayerView: UIView {
         return isSmallerScreen ? 40 : 60
     }()
 
-    private lazy var progressionViewBottomConstraint: NSLayoutConstraint = {
-        let progressionViewBottomConstraint = progressionView.bottomAnchor.constraint(equalTo: layoutGuide.bottomAnchor, constant: -progressionViewBottomConstant)
-        return progressionViewBottomConstraint
-    }()
+    private lazy var progressionViewBottomConstraint: NSLayoutConstraint = progressionView.bottomAnchor.constraint(equalTo: layoutGuide.bottomAnchor, constant: -progressionViewBottomConstant)
+
+    private lazy var progressionViewHeightConstraint: NSLayoutConstraint = progressionView.heightAnchor.constraint(equalToConstant: 70)
+
+    private lazy var thumbnailViewTopConstraint: NSLayoutConstraint = thumbnailView.topAnchor.constraint(equalTo: navigationBarView.bottomAnchor, constant: 35)
+
+    private lazy var controlsStackViewMinSpacing: CGFloat = 25.0
+    private lazy var controlsStackViewMaxSpacing: CGFloat = 50.0
 
     weak var delegate: AudioPlayerViewDelegate?
 
@@ -174,9 +209,14 @@ class AudioPlayerView: UIView {
         thumbnailImageView.image = delegate?.audioPlayerViewDelegateGetThumbnail(self)
         thumbnailImageView.clipsToBounds = true
     }
+    
+    func setupPlaybackSpeed() {
+        let defaultPlaybackSpeed = delegate?.audioPlayerViewDelegateGetPlaybackSpeed(self)
+        playbackSpeedButton.setTitle(String(format: "%.2fx", defaultPlaybackSpeed ?? 1.00), for: .normal)
+    }
 
     func setupBackgroundColor() {
-        backgroundView.backgroundColor = thumbnailImageView.image?.averageColor
+        backgroundView.backgroundColor = thumbnailImageView.image?.averageColor()
     }
 
     func setupLabels() {
@@ -218,23 +258,17 @@ class AudioPlayerView: UIView {
         ])
     }
 
-    func setupSliders() {
-        let brightnessControlView = delegate?.audioPlayerViewDelegateGetBrightnessSlider(self)
-        let volumeControlView = delegate?.audioPlayerViewDelegateGetVolumeSlider(self)
+    func setupSliders(with brightnessControlView: BrightnessControlView, and volumeControlView: VolumeControlView) {
+        thumbnailView.addSubview(brightnessControlView)
+        thumbnailView.addSubview(volumeControlView)
 
-        if let brightnessControlView = brightnessControlView,
-           let volumeControlView = volumeControlView {
-            thumbnailView.addSubview(brightnessControlView)
-            thumbnailView.addSubview(volumeControlView)
+        setupCommonSliderConstraints(for: brightnessControlView)
+        setupCommonSliderConstraints(for: volumeControlView)
 
-            setupCommonSliderConstraints(for: brightnessControlView)
-            setupCommonSliderConstraints(for: volumeControlView)
-
-            NSLayoutConstraint.activate([
-                brightnessControlView.leadingAnchor.constraint(equalTo: thumbnailView.leadingAnchor),
-                volumeControlView.trailingAnchor.constraint(equalTo: thumbnailView.trailingAnchor)
-            ])
-        }
+        NSLayoutConstraint.activate([
+            brightnessControlView.leadingAnchor.constraint(equalTo: thumbnailView.leadingAnchor, constant: 10),
+            volumeControlView.trailingAnchor.constraint(equalTo: thumbnailView.trailingAnchor, constant: -10)
+        ])
     }
 
     func updateLabels(title: String?, artist: String?, isQueueHidden: Bool) {
@@ -243,7 +277,9 @@ class AudioPlayerView: UIView {
             artistLabel.isHidden = false
 
             titleLabel.text = title
+            titleLabel.accessibilityValue = title
             artistLabel.text = artist
+            artistLabel.accessibilityValue = artist
         } else {
             titleLabel.isHidden = true
             artistLabel.isHidden = true
@@ -256,25 +292,40 @@ class AudioPlayerView: UIView {
     }
 
     func updateShuffleRepeatState(shuffleEnabled: Bool, repeatMode: VLCRepeatMode) {
-        let orangeColor = PresentationTheme.current.colors.orangeUI
+        var color = PresentationTheme.current.colors.orangeUI
 
         let shuffleIcon = shuffleEnabled ? UIImage(named: "iconShuffleOnLarge") : UIImage(named: "iconShuffleLarge")
         shuffleButton.setImage(shuffleIcon, for: .normal)
-        shuffleButton.tintColor = shuffleEnabled ? orangeColor : .white
+        shuffleButton.tintColor = shuffleEnabled ? color : .white
+        shuffleButton.accessibilityLabel = shuffleEnabled ? NSLocalizedString("SHUFFLE", comment: "") : NSLocalizedString("SHUFFLE_DISABLED", comment: "")
+        shuffleButton.accessibilityHint = shuffleEnabled ? NSLocalizedString("SHUFFLE_HINT", comment: "") : NSLocalizedString("SHUFFLE_OFF_HINT", comment: "")
 
+        var icon: UIImage?
+        var accessibilityLabel: String
+        var accessibilityHint: String
         switch repeatMode {
         case .doNotRepeat:
-            repeatButton.setImage(UIImage(named: "iconRepeatLarge"), for: .normal)
-            repeatButton.tintColor = .white
+            icon = UIImage(named: "iconRepeatLarge")
+            color = .white
+            accessibilityLabel = NSLocalizedString("MENU_REPEAT_DISABLED", comment: "")
+            accessibilityHint = NSLocalizedString("DO_NOT_REPEAT_HINT", comment: "")
         case .repeatCurrentItem:
-            repeatButton.setImage(UIImage(named: "iconRepeatOneOnLarge"), for: .normal)
-            repeatButton.tintColor = orangeColor
+            icon = UIImage(named: "iconRepeatOneOnLarge")
+            accessibilityLabel = NSLocalizedString("MENU_REPEAT_SINGLE", comment: "")
+            accessibilityHint = NSLocalizedString("REPEAT_HINT", comment: "")
         case .repeatAllItems:
-            repeatButton.setImage(UIImage(named: "iconRepeatOnLarge"), for: .normal)
-            repeatButton.tintColor = orangeColor
+            icon = UIImage(named: "iconRepeatOnLarge")
+            accessibilityLabel = NSLocalizedString("MENU_REPEAT_ALL", comment: "")
+            accessibilityHint = NSLocalizedString("REPEAT_ALL_HINT", comment: "")
         @unknown default:
             assertionFailure("AudioPlayerView: unhandled case.")
+            return
         }
+
+        repeatButton.setImage(icon, for: .normal)
+        repeatButton.tintColor = color
+        repeatButton.accessibilityLabel = accessibilityLabel
+        repeatButton.accessibilityHint = accessibilityHint
     }
 
     func setControlsEnabled(_ enabled: Bool) {
@@ -292,6 +343,39 @@ class AudioPlayerView: UIView {
 
         repeatButton.isEnabled = enabled
         repeatButton.alpha = enabled ? 1.0 : 0.5
+        
+        playbackSpeedButton.isEnabled = enabled
+        playbackSpeedButton.alpha = enabled ? 1.0 : 0.5
+    }
+
+    func updateConstraints(for orientation: UIDeviceOrientation) {
+        let isPad: Bool = UIDevice.current.userInterfaceIdiom == .pad
+
+        if orientation.isLandscape {
+            thumbnailViewTopConstraint.constant = 5
+            progressionViewBottomConstraint.constant = -5.0
+            progressionViewHeightConstraint.constant = 30
+            controlsStackView.spacing = isPad ? controlsStackViewMaxSpacing * 2 : controlsStackViewMaxSpacing
+        } else {
+            thumbnailViewTopConstraint.constant = 35
+            progressionViewBottomConstraint.constant = -progressionViewBottomConstant
+            progressionViewHeightConstraint.constant = 70
+            controlsStackView.spacing = isPad ? controlsStackViewMinSpacing * 2 : controlsStackViewMinSpacing
+        }
+
+        setNeedsLayout()
+        layoutIfNeeded()
+    }
+
+    func shouldDisableControls(_ disable: Bool) {
+        shuffleButton.isEnabled = !disable
+        previousButton.isEnabled = !disable
+        nextButton.isEnabled = !disable
+        repeatButton.isEnabled = !disable
+    }
+
+    func shouldDisplaySecondaryStackView(_ display: Bool) {
+        secondaryControlStackView.isHidden = !display
     }
 
     // MARK: - Private methods
@@ -354,13 +438,12 @@ class AudioPlayerView: UIView {
     }
 
     private func setupThumbnailView() {
-        let padding: CGFloat = 35.0
         thumbnailView.translatesAutoresizingMaskIntoConstraints = false
 
         addSubview(thumbnailView)
         NSLayoutConstraint.activate([
             thumbnailView.leadingAnchor.constraint(equalTo: layoutGuide.leadingAnchor),
-            thumbnailView.topAnchor.constraint(equalTo: navigationBarView.bottomAnchor, constant: padding),
+            thumbnailViewTopConstraint,
             thumbnailView.trailingAnchor.constraint(equalTo: layoutGuide.trailingAnchor),
         ])
 
@@ -413,31 +496,43 @@ class AudioPlayerView: UIView {
             playqueueView.leadingAnchor.constraint(equalTo: thumbnailView.leadingAnchor),
             playqueueView.topAnchor.constraint(equalTo: thumbnailView.topAnchor),
             playqueueView.trailingAnchor.constraint(equalTo: thumbnailView.trailingAnchor),
-            playqueueView.bottomAnchor.constraint(equalTo: thumbnailView.bottomAnchor)
+            playqueueView.bottomAnchor.constraint(equalTo: thumbnailView.bottomAnchor),
+            playqueueView.heightAnchor.constraint(equalTo: thumbnailView.heightAnchor, multiplier: 1)
         ])
     }
 
     private func setupControlsStackView() {
-        let padding: CGFloat = 50.0
         let topPadding: CGFloat = 20.0
-
-        controlsStackView.translatesAutoresizingMaskIntoConstraints = false
-        controlsStackView.alignment = .fill
-        controlsStackView.distribution = .fillEqually
-
-        addSubview(controlsStackView)
+        [controlsStackView, secondaryControlStackView].forEach {
+            $0.translatesAutoresizingMaskIntoConstraints = false
+            $0.alignment = .fill
+            $0.distribution = .equalCentering
+            
+            addSubview($0)
+        }
+        
         NSLayoutConstraint.activate([
             controlsStackView.topAnchor.constraint(equalTo: thumbnailView.bottomAnchor, constant: topPadding),
-            controlsStackView.leadingAnchor.constraint(equalTo: layoutGuide.leadingAnchor, constant: padding),
-            controlsStackView.trailingAnchor.constraint(equalTo: layoutGuide.trailingAnchor, constant: -padding),
+            controlsStackView.centerXAnchor.constraint(equalTo: centerXAnchor),
             controlsStackView.heightAnchor.constraint(equalToConstant: 50.0)
         ])
+        
+        NSLayoutConstraint.activate([
+            secondaryControlStackView.topAnchor.constraint(equalTo: controlsStackView.bottomAnchor, constant: topPadding/4),
+            secondaryControlStackView.centerXAnchor.constraint(equalTo: centerXAnchor),
+            secondaryControlStackView.heightAnchor.constraint(equalToConstant: 30.0)
+        ])
 
         controlsStackView.addArrangedSubview(shuffleButton)
         controlsStackView.addArrangedSubview(previousButton)
         controlsStackView.addArrangedSubview(playButton)
         controlsStackView.addArrangedSubview(nextButton)
         controlsStackView.addArrangedSubview(repeatButton)
+        
+        secondaryControlStackView.addArrangedSubview(playbackSpeedButton)
+
+        let displaySecondaryStackView: Bool = UserDefaults.standard.bool(forKey: kVLCPlayerShowPlaybackSpeedShortcut)
+        secondaryControlStackView.isHidden = !displaySecondaryStackView
     }
 
     private func setupProgressionView() {
@@ -449,10 +544,10 @@ class AudioPlayerView: UIView {
         addSubview(progressionView)
         NSLayoutConstraint.activate([
             progressionView.leadingAnchor.constraint(equalTo: layoutGuide.leadingAnchor, constant: padding),
-            progressionView.topAnchor.constraint(equalTo: controlsStackView.bottomAnchor, constant: padding),
+            progressionView.topAnchor.constraint(equalTo: secondaryControlStackView.bottomAnchor, constant: padding),
             progressionView.trailingAnchor.constraint(equalTo: layoutGuide.trailingAnchor, constant: -padding),
             progressionViewBottomConstraint,
-            progressionView.heightAnchor.constraint(equalToConstant: 70)
+            progressionViewHeightConstraint
         ])
     }
 
@@ -483,4 +578,14 @@ class AudioPlayerView: UIView {
     @objc func handleRepeatButton(_ sender: Any) {
         delegate?.audioPlayerViewDelegateDidTapRepeatButton(self)
     }
+    
+    @objc func handlePlaybackSpeedButton(_ sender: Any) {
+        delegate?.audioPlayerViewDelegateDidTapPlaybackSpeedButton(self)
+    }
+    
+    @objc func handleLongPressPlaybackSpeedButton(_ sender: UILongPressGestureRecognizer) {
+        if sender.state == .began {
+            delegate?.audioPlayerViewDelegateDidLongPressPlaybackSpeedButton(self)
+        }
+    }
 }
