@@ -911,6 +911,13 @@ extension PlayerViewController: VLCPlaybackServiceDelegate {
         if currentState == .opening {
             applyCustomEqualizerProfileIfNeeded()
         }
+
+        if currentState == .stopped {
+            moreOptionsActionSheet.resetPlaybackSpeed()
+            mediaMoreOptionsActionSheetHideIcon(for: .playbackSpeed)
+            moreOptionsActionSheet.resetSleepTimer()
+            mediaMoreOptionsActionSheetHideIcon(for: .sleepTimer)
+        }
     }
 
     func showStatusMessage(_ statusMessage: String) {
@@ -955,7 +962,7 @@ extension PlayerViewController: MediaMoreOptionsActionSheetDelegate {
             showIcon(button: optionsNavigationBar.abRepeatMarksButton)
             break
         default:
-            assertionFailure("AudioPlayerViewController: Invalid option.")
+            assertionFailure("PlayerViewController: Invalid option.")
         }
     }
 
@@ -977,7 +984,7 @@ extension PlayerViewController: MediaMoreOptionsActionSheetDelegate {
             hideIcon(button: optionsNavigationBar.abRepeatMarksButton)
             break
         default:
-            assertionFailure("AudioPlayerViewController: Invalid option.")
+            assertionFailure("PlayerViewController: Invalid option.")
         }
     }
 
