@@ -46,6 +46,11 @@ typedef NS_ENUM(NSInteger, VLCPlayerScanState)
 
 @property (nonatomic) BOOL playbackUIShouldHide;
 
+// 360 Support
+@property (nonatomic) CGPoint projectionLocation;
+@property (nonatomic) CGFloat yaw;
+@property (nonatomic) CGFloat pitch;
+
 @end
 
 @implementation VLCFullscreenMovieTVViewController
@@ -84,6 +89,9 @@ typedef NS_ENUM(NSInteger, VLCPlayerScanState)
 
     self.bufferingLabel.text = NSLocalizedString(@"PLEASE_WAIT", nil);
 
+    self.pitch = 0;
+    self.yaw = 0;
+
     _disabledIdleTimer = NO;
 
     NSMutableSet<UIGestureRecognizer *> *simultaneousGestureRecognizers = [NSMutableSet set];
@@ -104,7 +112,7 @@ typedef NS_ENUM(NSInteger, VLCPlayerScanState)
     menuTapGestureRecognizer.delegate = self;
     [self.view addGestureRecognizer:menuTapGestureRecognizer];
 
-    // IR only recognizer
+    //  IR only recognizer
     UITapGestureRecognizer *upArrowRecognizer = [[VLCIRTVTapGestureRecognizer alloc] initWithTarget:self action:@selector(handleIRPressUp)];
     upArrowRecognizer.allowedPressTypes = @[@(UIPressTypeUpArrow)];
     [self.view addGestureRecognizer:upArrowRecognizer];
@@ -191,6 +199,7 @@ typedef NS_ENUM(NSInteger, VLCPlayerScanState)
     [vpc recoverDisplayedMetadata];
     vpc.videoOutputView = nil;
     vpc.videoOutputView = self.movieView;
+    [vpc disableSubtitlesIfNeeded];
 }
 
 - (void)viewWillDisappear:(BOOL)animated
@@ -250,6 +259,7 @@ typedef NS_ENUM(NSInteger, VLCPlayerScanState)
     }
 
     switch (panGestureRecognizer.state) {
+        case UIGestureRecognizerStateBegan:
         case UIGestureRecognizerStateCancelled:
         case UIGestureRecognizerStateFailed:
             return;
@@ -271,9 +281,17 @@ typedef NS_ENUM(NSInteger, VLCPlayerScanState)
                 return;
             }
         } else if (translation.y > 200.0) {
-            panGestureRecognizer.enabled = NO;
-            panGestureRecognizer.enabled = YES;
-            [self showInfoVCIfNotScrubbing];
+            if ([[VLCPlaybackService sharedInstance] currentMediaIs360Video]) {
+                if ([VLCPlaybackService sharedInstance].mediaPlayerState == VLCMediaPlayerStatePaused) {
+                    panGestureRecognizer.enabled = NO;
+                    panGestureRecognizer.enabled = YES;
+                    [self showInfoVCIfNotScrubbing];
+                }
+            } else {
+                panGestureRecognizer.enabled = NO;
+                panGestureRecognizer.enabled = YES;
+                [self showInfoVCIfNotScrubbing];
+            }
             return;
         } else {
             return;
@@ -302,8 +320,8 @@ typedef NS_ENUM(NSInteger, VLCPlayerScanState)
                           delay:0.0
                         options:UIViewAnimationOptionAllowUserInteraction | UIViewAnimationOptionBeginFromCurrentState
                      animations:^{
-                         bar.scrubbingFraction = scrubbingFraction;
-                     }
+        bar.scrubbingFraction = scrubbingFraction;
+    }
                      completion:nil];
     [self updateTimeLabelsForScrubbingFraction:scrubbingFraction];
 }
@@ -332,6 +350,7 @@ typedef NS_ENUM(NSInteger, VLCPlayerScanState)
         [vpc playPause];
     }
 }
+
 - (void)menuButtonPressed:(UITapGestureRecognizer *)recognizer
 {
     VLCTransportBar *bar = self.transportBar;
@@ -449,6 +468,33 @@ typedef NS_ENUM(NSInteger, VLCPlayerScanState)
     [self scanForwardPrevious];
 }
 
+- (void)updateProjection:(VLCSiriRemoteGestureRecognizer *)recognizer
+{
+    CGPoint newLocationInView = [recognizer locationInView:self.view];
+
+    CGFloat diffX = newLocationInView.x - self.projectionLocation.x;
+    CGFloat diffY = newLocationInView.y - self.projectionLocation.y;
+    self.projectionLocation = newLocationInView;
+
+    CGSize screenPixelSize = CGSizeMake([UIScreen mainScreen].bounds.size.width, [UIScreen mainScreen].bounds.size.height);
+
+    CGFloat diffYaw = 85 * -diffX / screenPixelSize.width;
+    CGFloat diffPitch = 85 * -diffY / screenPixelSize.width;
+
+    [self applyYaw:diffYaw pitch:diffPitch];
+}
+
+- (void)applyYaw:(CGFloat)yaw pitch:(CGFloat)pitch
+{
+    self.yaw += yaw;
+    self.pitch = self.pitch  + MIN(MAX(pitch, -90), 90);
+
+    [self setPitch: self.pitch + pitch];
+
+    VLCPlaybackService *vpc = [VLCPlaybackService sharedInstance];
+    [vpc updateViewpoint:self.yaw pitch:self.pitch roll:0 fov:85 absolute:YES];
+}
+
 - (void)handleSiriRemote:(VLCSiriRemoteGestureRecognizer *)recognizer
 {
     VLCPlaybackService *vpc = [VLCPlaybackService sharedInstance];
@@ -499,43 +545,58 @@ typedef NS_ENUM(NSInteger, VLCPlayerScanState)
     VLCTransportBarHint hint = self.transportBar.hint;
     switch (recognizer.state) {
         case UIGestureRecognizerStateBegan:
+            if ([[VLCPlaybackService sharedInstance] currentMediaIs360Video]) {
+                _projectionLocation = [recognizer locationInView:self.view];
+                break;
+            }
         case UIGestureRecognizerStateChanged:
-            if (recognizer.isLongPress) {
-                if (!self.isSeekable && recognizer.touchLocation == VLCSiriRemoteTouchLocationRight) {
-                    [self setScanState:VLCPlayerScanStateForward2];
-                    return;
-                }
-            } else {
-                if (self.canJump) {
-                    switch (recognizer.touchLocation) {
-                        case VLCSiriRemoteTouchLocationLeft:
-                            hint = VLCTransportBarHintJumpBackward10;
-                            break;
-                        case VLCSiriRemoteTouchLocationRight:
-                            hint = VLCTransportBarHintJumpForward10;
-                            break;
-                        default:
-                            hint = VLCTransportBarHintNone;
-                            break;
+            if (![[VLCPlaybackService sharedInstance] currentMediaIs360Video]) {
+                if (recognizer.isLongPress) {
+                    if (!self.isSeekable && recognizer.touchLocation == VLCSiriRemoteTouchLocationRight) {
+                        [self setScanState:VLCPlayerScanStateForward2];
+                        return;
                     }
                 } else {
-                    hint = VLCTransportBarHintNone;
+                    if (self.canJump) {
+                        switch (recognizer.touchLocation) {
+                            case VLCSiriRemoteTouchLocationLeft:
+                                hint = VLCTransportBarHintJumpBackward10;
+                                break;
+                            case VLCSiriRemoteTouchLocationRight:
+                                hint = VLCTransportBarHintJumpForward10;
+                                break;
+                            default:
+                                hint = VLCTransportBarHintNone;
+                                break;
+                        }
+                    } else {
+                        hint = VLCTransportBarHintNone;
+                    }
                 }
             }
             break;
         case UIGestureRecognizerStateEnded:
-            if (recognizer.isClick && !recognizer.isLongPress) {
-                [self handleSiriPressUpAtLocation:recognizer.touchLocation];
+            if (![[VLCPlaybackService sharedInstance] currentMediaIs360Video]) {
+                if (recognizer.isClick && !recognizer.isLongPress) {
+                    [self handleSiriPressUpAtLocation:recognizer.touchLocation];
+                }
+                [self setScanState:VLCPlayerScanStateNone];
             }
-            [self setScanState:VLCPlayerScanStateNone];
             break;
         case UIGestureRecognizerStateCancelled:
-            hint = VLCTransportBarHintNone;
-            [self setScanState:VLCPlayerScanStateNone];
+            if (![[VLCPlaybackService sharedInstance] currentMediaIs360Video]) {
+                hint = VLCTransportBarHintNone;
+                [self setScanState:VLCPlayerScanStateNone];
+            }
             break;
         default:
             break;
     }
+
+    if (!self.transportBar.isScrubbing) {
+        [self updateProjection:recognizer];
+    }
+
     self.transportBar.hint = self.isSeekable ? hint : VLCPlayerScanStateNone;
 }
 
@@ -852,8 +913,8 @@ static const NSInteger VLCJumpInterval = 10000; // 10 seconds
     CGFloat alpha = visible ? 1.0 : 0.0;
     [UIView animateWithDuration:duration
                      animations:^{
-                         self.bottomOverlayView.alpha = alpha;
-                     }];
+        self.bottomOverlayView.alpha = alpha;
+    }];
 }
 
 
@@ -889,7 +950,7 @@ static const NSInteger VLCJumpInterval = 10000; // 10 seconds
                       isPlaying:(BOOL)isPlaying
 currentMediaHasTrackToChooseFrom:(BOOL)currentMediaHasTrackToChooseFrom
         currentMediaHasChapters:(BOOL)currentMediaHasChapters
-          forPlaybackService:(VLCPlaybackService *)playbackService
+             forPlaybackService:(VLCPlaybackService *)playbackService
 {
 
     [self updateActivityIndicatorForState:currentState];
