@@ -228,7 +228,36 @@ class QueueViewController: UIViewController {
 
         topConstraint?.constant = topConstraintConstant
         reload()
+    }
 
+    @objc init(medialibraryService: MediaLibraryService) {
+        self.medialibraryService = medialibraryService
+        super.init(nibName: nil, bundle: nil)
+        view.alpha = 0.0
+    }
+
+    @objc func show() {
+        UIView.animate(withDuration: animationDuration, animations: {
+            self.view.alpha = 1.0
+            self.darkOverlayView.isHidden = false
+        }) { _ in
+            self.scrollToCurrentlyPlaying()
+        }
+    }
+
+    @objc func hide() {
+        if let parent = parent as? VLCPlayerDisplayController {
+            guard !parent.hintingPlayqueue else {
+                return
+            }
+        }
+        UIView.animate(withDuration: animationDuration, animations: {
+            self.view.alpha = 0.0
+            self.darkOverlayView.isHidden = true
+        })
+    }
+
+    private func scrollToCurrentlyPlaying() {
         guard let currentMedia = playbackService.currentlyPlayingMedia else {
             return
         }
@@ -251,31 +280,6 @@ class QueueViewController: UIViewController {
         }
     }
 
-    @objc init(medialibraryService: MediaLibraryService) {
-        self.medialibraryService = medialibraryService
-        super.init(nibName: nil, bundle: nil)
-        view.alpha = 0.0
-    }
-
-    @objc func show() {
-        UIView.animate(withDuration: animationDuration, animations: {
-            self.view.alpha = 1.0
-            self.darkOverlayView.isHidden = false
-        })
-    }
-
-    @objc func hide() {
-        if let parent = parent as? VLCPlayerDisplayController {
-            guard !parent.hintingPlayqueue else {
-                return
-            }
-        }
-        UIView.animate(withDuration: animationDuration, animations: {
-            self.view.alpha = 0.0
-            self.darkOverlayView.isHidden = true
-        })
-    }
-
     required init?(coder aDecoder: NSCoder) {
         fatalError("init(coder:) has not been implemented")
     }
